﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.SocialPlatforms.Impl;
using System;

public class AsWeAreController : MonoBehaviour {

    public const int MAX_ROUNDS = 15;
    public const int GUARANTEED_FUN_ROUNDS = 3;
    public const int GUARANTEED_DEEP_ROUNDS = 1;
    public const float DEEP_RATIO = 1.0f;
    public const string CONTINUE_TEXT = "Continue";
    public const string START_TEXT = "[dddd22]Start game";
    public const string RESET_TEXT = "Play again?";

    public static readonly string[] FUN_INSTRUCTIONS = 
    {
        "Do 10 jumping jacks",
        "Do 5 push-ups",
        "Do 5 sit-ups",
        "I spy with my little eye, something beginning with Z…",
        "Run the perimeter of the room.",
        "Hug all the other players",
        "Trade a piece of clothing with your teammate",
        "Balance a book on your head for ten seconds",
        "Win a staring contest with an opponent",
        "Win an arm wrestling contest with an opponent",
        "Recite the alphabet backwards",
        "Make an opponent laugh",
        "Poke the back of an opponent's head",
        "Win a thumb war with an opponent",
        "Invent and define a new onomatopoeia",
        "Lift an opponent for five seconds",
        "Touch your toes",
        "Sing the chorus to your favorite song",
        "Do an impression of an opponent",
        "Do an impression of a teammate",
        "Assume the fetal position",
        "Win a dance contest by popular vote",
        "Win a breath holding contest with an opponent",
        "Win a chicken impersonation contest by popular vote"
    };

    public static readonly string[] DEEP_INSTRUCTIONS = 
    {
        "Describe the last dream you had",
        "Tell us how you describe yourself to others",
        "Describe your own personality",
        "Describe how your last relationship ended",
        "Describe your first relationship",
        "Tell us what you think of dogs",
        "Tell us your favorite food and why",
        "Tell us your greatest regret",
        "Describe your earliest memory",
        "Tell us something about your childhood that makes you sad",
        "Tell us something about your childhood that makes you happy",
        "Tell us an opinion you hold but know you shouldn’t",
        "Tell us the cruelest thing you've ever done",
        "Describe how your opinion of the other players has changed during this game",
        "Tell us your guiltiest pleasure and why",
        "Describe the most extravagant thing you've ever bought",
        "Tell us would you most like to live and why",
        "Describe where you see yourself in five years",
        "Describe where you see yourself in twenty years",
        "Tell us what you thought your life would be like by now when you were a child",
        "Describe what would you change about who you are",
        "Describe the circumstances around your first kiss",
        "Tell us when you last finished a book",
        "Tell us your greatest talent",
        "Tell us your greatest weakness",
        "Tell us your greatest vice",
        "Describe something you like about an opponent's personality",
        "Describe something you dislike about a team mate's personality", 
        "Tell us your New Year’s Resolutions from this year and if you are still following them",
        "Describe what your job would be if money was irrelevant",
        "Tell us a lie you tell yourself",
        "Tell us what you think of religion",
        "Describe one thing you would change about the world and why",
        "Tell us where you would most like to travel to and why",
        "Tell us your least favorite place you've ever been and why"
    };

    public static readonly string[] PREINSTRUCTIONS =
    {
        "As We Are is a game for two teams of friends and acquaintances.\n" +
        "Please split into two teams, then press Z or M to continue.",
        "Each round this window will display a challenge.\n" +
        "Press your team's key (Z or M) after one of your teammates has " +
        "completed the challenge to earn a point for your team.",
        "There will be " + MAX_ROUNDS + " challenges. The first team to " + 
        "complete each challenge wins the point for that round.\n\n" +
        "Press Z or M to begin."
    };

    public const string WIN_INSTR_1 = "Team ";
    public const string WIN_INSTR_2 = " has the most points. Hooray for them.\n" +
        "Play again?";
    public const string TIE_INSTR = "It's a tie! Isn't that nice?";

    public UILabel[] buttonLabels;
	public UILabel[] scoreLabels;
    public UIButton[] buttons;
    public ParticleSystem[] particles;
    public UILabel instructions;
    public UILabel roundLabel;
    public TweenAlpha instructionsTween;

    public AudioClip selectSound;
    public AudioClip scoreSound;
    public AudioClip gameOverMusic;

    private string[] defButtonLabelText;

    int[] score;

    bool gameStarted = false;
    bool gameOver = false;
    int round = 0;
    int instructionsRound = 0;

    List<string> funText;
    List<string> deepText;

    AudioSource audioSource;

    void Awake()
    {
        audioSource = GetComponent<AudioSource>();
    }

	// Use this for initialization
	void Start () 
    {
        defButtonLabelText = new string[buttonLabels.Length];
        score = new int[scoreLabels.Length];

        for (int i = 0; i < buttonLabels.Length; i++)
        {
            defButtonLabelText[i] = buttonLabels[i].text;
        }
        for (int i = 0; i < buttons.Length; i++)
        {
            int team = i;
            EventDelegate.Add(buttons[i].onClick, () =>
            {
                ButtonClicked(team);
            });
        }
        EventDelegate.Add(instructionsTween.onFinished, OnFinished);

        funText = new List<string>(FUN_INSTRUCTIONS);
        deepText = new List<string>(DEEP_INSTRUCTIONS);

        GetNewText();
	}
	
	// Update is called once per frame
	void Update () 
    {
        roundLabel.text = "" + round + "/" + MAX_ROUNDS;
        for (int i = 0; i < scoreLabels.Length; i++)
        {
            scoreLabels[i].text = score[i].ToString();
        }

        if (gameStarted && !gameOver)
        {
            for (int i = 0; i < buttonLabels.Length; i++)
            {
                buttonLabels[i].text = defButtonLabelText[i];
            }
        }
        else
        {
            string preText = CONTINUE_TEXT;
            if (gameOver)
            {
                preText = RESET_TEXT;
            }
            else if (instructionsRound >= PREINSTRUCTIONS.Length-1)
            {
                preText = START_TEXT;
            }
            for (int i = 0; i < buttonLabels.Length; i++)
            {
                buttonLabels[i].text = preText;
            }
        }
	}

    public void ButtonClicked(int team)
    {
        if (!instructionsTween.enabled)
        {
            if (gameOver)
            {
                Application.LoadLevel(0);
            }

            instructionsTween.enabled = true;
            instructionsTween.PlayForward();

            if (gameStarted)
            {
                Score(team);
            }
            audioSource.PlayOneShot(selectSound);
        }
    }

    public void OnFinished ()
    {
        instructionsTween.ResetToBeginning();
        AdvanceRound();
        GetNewText();
    }

    public void Score(int team)
    {
        score[team]++;
        particles[team].gameObject.SetActive(true);
        particles[team].Stop();
        particles[team].Play();
        audioSource.PlayOneShot(scoreSound);
    }

    public void AdvanceRound()
    {
        if (!gameStarted)
        {
            instructionsRound++;
            if (instructionsRound == PREINSTRUCTIONS.Length)
            {
                gameStarted = true;
            }
        }

        if (gameStarted)
        {
            if (round >= MAX_ROUNDS)
            {
                gameOver = true;
                audioSource.clip = gameOverMusic;
                audioSource.Play();
            }
            else
            {
                round++;
            }
        }
    }

    public void GetNewText()
    {
        string newText;
        if (gameOver)
        {
            if (score[0] == score[1])
            {
                newText = TIE_INSTR;
            }
            else
            {
                int winningTeam = score[0] > score[1] ? 0 : 1;
                newText = WIN_INSTR_1 + (winningTeam+1) + WIN_INSTR_2;
            }
        }
        else if (!gameStarted)
        {
            newText = PREINSTRUCTIONS[instructionsRound];
        }
        else
        {
            newText = GetRoundText();
        }
        instructions.text = newText;
    }

    public string GetRoundText()
    {
        bool useFun;
        if (round <= GUARANTEED_FUN_ROUNDS)
        {
            useFun = true;
        }
        else if (round >= MAX_ROUNDS - GUARANTEED_DEEP_ROUNDS)
        {
            useFun = false;
        }
        else
        {
            float randomRounds = MAX_ROUNDS - GUARANTEED_FUN_ROUNDS - GUARANTEED_DEEP_ROUNDS;
            float randomSoFar = round - GUARANTEED_FUN_ROUNDS;
            float deepChance = (randomSoFar*DEEP_RATIO) / randomRounds;
            float roll = UnityEngine.Random.value;
            useFun = (roll > deepChance);
        }

        List<string> pickFrom = useFun ? funText : deepText;
        int index = UnityEngine.Random.Range(0, pickFrom.Count);
        string toRet = pickFrom[index];
        pickFrom.RemoveAt(index);
        return toRet;
    }
}
